﻿
Public Class clsObject

    Private picObject As PictureBox

    Public rectObject As Rectangle

    Private blnClick As Boolean = False
    Private blnMove As Boolean = False

    Private intPrevX As Integer
    Private intPrevY As Integer
    Private intNode As Integer = 5

    Private intSelNode As NodePos = NodePos.None

    Private Enum NodePos

        TopLeft
        TopMiddle
        TopRight

        BottomLeft
        BottomMiddle
        BottomRight

        LeftMiddle

        RightMiddle

        None

    End Enum

    Public Sub New(ByVal rctTemp As Rectangle)

        rectObject = rctTemp

        blnClick = False

    End Sub

    Public Sub Create(ByVal g As Graphics)

        g.DrawRectangle(New Pen(Color.Green), rectObject)

        For Each npPos As NodePos In [Enum].GetValues(GetType(NodePos))

            g.DrawRectangle(New Pen(Color.Green), GetObject(npPos))

        Next

    End Sub

    Public Sub AddPicEvents(ByVal pic As PictureBox)

        Me.picObject = pic

        AddHandler picObject.MouseDown, AddressOf picObject_MouseDown
        AddHandler picObject.MouseMove, AddressOf picObject_MouseMove
        AddHandler picObject.MouseUp, AddressOf picObject_MouseUp
        AddHandler picObject.Paint, AddressOf picObject_Paint

    End Sub

    Private Sub picObject_Paint(ByVal sender As Object, ByVal e As PaintEventArgs)

        Try

            Create(e.Graphics)

        Catch ex As Exception

            MessageBox.Show(ex.Message)

        End Try

    End Sub

    Private Sub picObject_MouseDown(ByVal sender As Object, ByVal e As MouseEventArgs)

        blnClick = True

        intSelNode = NodePos.None

        intSelNode = SelectableNode(e.Location)

        If rectObject.Contains(New Point(e.X, e.Y)) Then

            blnMove = True

        End If

        intPrevX = e.X
        intPrevY = e.Y

    End Sub

    Private Sub picObject_MouseUp(ByVal sender As Object, ByVal e As MouseEventArgs)

        blnClick = False
        blnMove = False

    End Sub

    Private Sub picObject_MouseMove(ByVal sender As Object, ByVal e As MouseEventArgs)

        ChangeCursor(e.Location)

        If blnClick = False Then

            Return

        End If

        Dim rctTemp As Rectangle = rectObject

        Select Case intSelNode

            Case NodePos.TopLeft

                rectObject.X += e.X - intPrevX
                rectObject.Width -= e.X - intPrevX
                rectObject.Y += e.Y - intPrevY
                rectObject.Height -= e.Y - intPrevY

            Case NodePos.LeftMiddle

                rectObject.X += e.X - intPrevX
                rectObject.Width -= e.X - intPrevX

            Case NodePos.BottomLeft

                rectObject.Width -= e.X - intPrevX
                rectObject.X += e.X - intPrevX
                rectObject.Height += e.Y - intPrevY

            Case NodePos.BottomMiddle

                rectObject.Height += e.Y - intPrevY

            Case NodePos.TopRight

                rectObject.Width += e.X - intPrevX
                rectObject.Y += e.Y - intPrevY
                rectObject.Height -= e.Y - intPrevY

            Case NodePos.BottomRight

                rectObject.Width += e.X - intPrevX
                rectObject.Height += e.Y - intPrevY

            Case NodePos.RightMiddle

                rectObject.Width += e.X - intPrevX

            Case NodePos.TopMiddle

                rectObject.Y += e.Y - intPrevY
                rectObject.Height -= e.Y - intPrevY

            Case Else

                If blnMove Then

                    rectObject.X = rectObject.X + e.X - intPrevX
                    rectObject.Y = rectObject.Y + e.Y - intPrevY

                End If

        End Select

        intPrevX = e.X
        intPrevY = e.Y

        If rectObject.Width < 5 OrElse rectObject.Height < 5 Then

            rectObject = rctTemp

        End If

        ObjHitTest()

        picObject.Invalidate()

    End Sub

    Private Sub ObjHitTest()

        If rectObject.X < 0 Then rectObject.X = 0
        If rectObject.Y < 0 Then rectObject.Y = 0

        If rectObject.Width <= 0 Then rectObject.Width = 1
        If rectObject.Height <= 0 Then rectObject.Height = 1

        If rectObject.X + rectObject.Width > picObject.Width Then

            rectObject.Width = picObject.Width - rectObject.X - 1

        End If

        If rectObject.Y + rectObject.Height > picObject.Height Then

            rectObject.Height = picObject.Height - rectObject.Y - 1

        End If

    End Sub

    Private Function SizeNode(ByVal x As Integer, ByVal y As Integer) As Rectangle

        Return New Rectangle(x - intNode / 2, y - intNode / 2, intNode, intNode)

    End Function

    Private Function GetObject(ByVal pos As NodePos) As Rectangle

        Select Case pos

            Case NodePos.TopLeft

                Return SizeNode(rectObject.X, rectObject.Y)

            Case NodePos.TopMiddle

                Return SizeNode(rectObject.X + rectObject.Width / 2, rectObject.Y)

            Case NodePos.TopRight

                Return SizeNode(rectObject.X + rectObject.Width, rectObject.Y)

            Case NodePos.BottomLeft

                Return SizeNode(rectObject.X, rectObject.Y + rectObject.Height)

            Case NodePos.BottomMiddle

                Return SizeNode(rectObject.X + rectObject.Width / 2, rectObject.Y + rectObject.Height)

            Case NodePos.BottomRight

                Return SizeNode(rectObject.X + rectObject.Width, rectObject.Y + rectObject.Height)

            Case NodePos.LeftMiddle

                Return SizeNode(rectObject.X, rectObject.Y + +rectObject.Height / 2)

            Case NodePos.RightMiddle

                Return SizeNode(rectObject.X + rectObject.Width, rectObject.Y + rectObject.Height / 2)

            Case Else

                Return New Rectangle()

        End Select

    End Function

    Private Function SelectableNode(ByVal pnt As Point) As NodePos

        For Each pos As NodePos In [Enum].GetValues(GetType(NodePos))

            If GetObject(pos).Contains(pnt) Then

                Return pos

            End If

        Next

        Return NodePos.None

    End Function

    Private Sub ChangeCursor(ByVal pnt As Point)

        picObject.Cursor = GetCursor(SelectableNode(pnt))

    End Sub

    Private Function GetCursor(ByVal p As NodePos) As Cursor

        Select Case p

            Case NodePos.TopLeft

                Return Cursors.SizeNWSE

            Case NodePos.TopMiddle

                Return Cursors.SizeNS

            Case NodePos.TopRight

                Return Cursors.SizeNESW

            Case NodePos.BottomLeft

                Return Cursors.SizeNESW

            Case NodePos.BottomMiddle

                Return Cursors.SizeNS

            Case NodePos.BottomRight

                Return Cursors.SizeNWSE

            Case NodePos.LeftMiddle

                Return Cursors.SizeWE

            Case NodePos.RightMiddle

                Return Cursors.SizeWE

            Case Else

                Return Cursors.[Default]

        End Select

    End Function

End Class


